@@ -18,6 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#define _XOPEN_SOURCE	/* strptime proto, but this hides other prototypes */
+#define _GNU_SOURCE	/* get all the other prototypes */
+
 #include "config.h"
 #include "actions.h"
 #include "i18n.h"
@@ -28,12 +31,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-/* we need these for timestamps of debugging messages */
 #include <time.h>
 #include <sys/time.h>
 
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-setting.h>
+#include <gphoto2/globals.h>
 
 #ifdef HAVE_AA
 #  include "gphoto2-cmd-capture.h"
@@ -88,18 +91,38 @@ int
 num_files_action (GPParams *p)
 {
 	CameraList *list;
-	int n;
+	int count, filecount;
 
 	CR (gp_list_new (&list));
 	CL (gp_camera_folder_list_files (p->camera, p->folder,
 					 list, p->context), list);
-	CL (n = gp_list_count (list), list);
-	gp_list_free (list);
-	if (p->quiet)
-		fprintf (stdout, "%i\n", n);
+	CL (count = gp_list_count (list), list);
+
+	if (p->flags & FLAGS_NEW) {
+		int i;
+		const char *name;
+
+		filecount = 0;
+		for (i = 0; i < count; i++) {
+			CameraFileInfo info;
+
+			CL (gp_list_get_name (list, i, &name), list);
+			CR (gp_camera_file_get_info (p->camera, p->folder,
+						     name, &info, p->context));
+			if (info.file.fields & GP_FILE_INFO_STATUS &&
+			    info.file.status != GP_FILE_STATUS_DOWNLOADED)
+				filecount++;
+		}
+	}
 	else
-		fprintf (stdout, _("Number of files in "
-				   "folder '%s': %i\n"), p->folder, n);
+	  filecount = count;
+
+	gp_list_free (list);
+	if (p->flags & FLAGS_QUIET)
+		printf ("%i\n", filecount);
+	else
+		printf (_("Number of files in folder '%s': %i\n"),
+			p->folder, filecount);
 
 	return (GP_OK);
 }
@@ -120,16 +143,16 @@ list_folders_action (GPParams *p)
 	switch (count) {
         case 0:
                 printf (_("There are no folders in folder '%s'."), p->folder);
-                printf ("\n");
+                putchar ('\n');
                 break;
         case 1:
                 printf (_("There is one folder in folder '%s':"), p->folder);
-                printf ("\n");
+                putchar ('\n');
                 break;
         default:
                 printf (_("There are %i folders in folder '%s':"),
 			 count, p->folder);
-                printf ("\n");
+                putchar ('\n');
                 break;
 	}
 	for (i = 0; i < count; i++) {
@@ -144,7 +167,7 @@ int
 list_files_action (GPParams *p)
 {
 	CameraList *list;
-	int count;
+	int count, filecount;
 	const char *name;
 	unsigned int i;
 
@@ -152,21 +175,35 @@ list_files_action (GPParams *p)
 	CL (gp_camera_folder_list_files (p->camera, p->folder, list,
 					 p->context), list);
 	CL (count = gp_list_count (list), list);
-	switch (count) {
+	if (p->flags & FLAGS_NEW) {
+		filecount = 0;
+		for (i = 0; i < count; i++) {
+			CameraFileInfo info;
+
+			CL (gp_list_get_name (list, i, &name), list);
+			CR (gp_camera_file_get_info (p->camera, p->folder,
+						     name, &info, p->context));
+			if (info.file.fields & GP_FILE_INFO_STATUS &&
+			    info.file.status != GP_FILE_STATUS_DOWNLOADED)
+				filecount++;
+		}
+	}
+	else
+	  filecount = count;
+
+	switch (filecount) {
 	case 0:
-		fprintf (stdout, _("There are no files in folder '%s'."),
-			 p->folder);
-		fputc ('\n', stdout);
+		printf (_("There are no files in folder '%s'."), p->folder);
+                putchar ('\n');
 		break;
 	case 1:
-		fprintf (stdout, _("There is one file in folder '%s':"),
-			 p->folder);
-		fputc ('\n', stdout);
+		printf (_("There is one file in folder '%s':"), p->folder);
+                putchar ('\n');
 		break;
 	default:
-		fprintf (stdout, _("There are %i files in folder '%s':"),
-			 count, p->folder);
-		fputc ('\n', stdout);
+		printf (_("There are %i files in folder '%s':"),
+			filecount, p->folder);
+                putchar ('\n');
 		break;
 	}
 	for (i = 0; i < count; i++) {
@@ -215,7 +252,7 @@ print_info_action (GPParams *p, const char *filename)
 				printf (_("delete"));
 			else
 				printf (_("none"));
-			printf ("\n");
+			putchar ('\n');
 		}
 		if (info.file.fields & GP_FILE_INFO_MTIME)
 			printf (_("  Time:        %s"),
@@ -256,9 +293,21 @@ print_info_action (GPParams *p, const char *filename)
 int
 print_file_action (GPParams *p, const char *filename)
 {
-	static int x=0;
+	static int x = 0;
 
-	if (p->quiet)
+	if (p->flags & FLAGS_NEW) {
+		CameraFileInfo info;
+		
+		CR (gp_camera_file_get_info (p->camera, p->folder,
+					     filename, &info, p->context));
+		if (info.file.fields & GP_FILE_INFO_STATUS &&
+		    info.file.status == GP_FILE_STATUS_DOWNLOADED) {
+			x++;
+			return (GP_OK);
+		}
+	}
+
+	if (p->flags & FLAGS_QUIET)
 		printf ("\"%s\"\n", filename);
 	else {
 		CameraFileInfo info;
@@ -267,8 +316,8 @@ print_file_action (GPParams *p, const char *filename)
 		    printf("#%-5i %-27s", x+1, filename);
 		    if (info.file.fields & GP_FILE_INFO_PERMISSIONS) {
 			printf("%s%s",
-				(info.file.permissions & GP_FILE_PERM_READ) ? "r" : "-",
-				(info.file.permissions & GP_FILE_PERM_DELETE) ? "d" : "-");
+			       (info.file.permissions & GP_FILE_PERM_READ) ? "r" : "-",
+			       (info.file.permissions & GP_FILE_PERM_DELETE) ? "d" : "-");
 		    }
 		    if (info.file.fields & GP_FILE_INFO_SIZE)
 			printf(" %5ld KB", (info.file.size+1023) / 1024);
@@ -277,10 +326,9 @@ print_file_action (GPParams *p, const char *filename)
 			printf(" %4dx%-4d", info.file.width, info.file.height);
 		    if (info.file.fields & GP_FILE_INFO_TYPE)
 			printf(" %s", info.file.type);
-			printf("\n");
-		} else {
+		    putchar ('\n');
+		} else
 		    printf("#%-5i %s\n", x+1, filename);
-		}
 	}
 	x++;
 	return (GP_OK);
@@ -289,36 +337,36 @@ print_file_action (GPParams *p, const char *filename)
 int
 save_file_action (GPParams *p, const char *filename)
 {
-	return (save_file_to_file (p->camera, p->context, p->folder, filename,
-				   GP_FILE_TYPE_NORMAL));
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_NORMAL));
 }
 
 int
 save_exif_action (GPParams *p, const char *filename)
 {
-	return (save_file_to_file (p->camera, p->context, p->folder, filename,
-				   GP_FILE_TYPE_EXIF));
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_EXIF));
 }
 
 int
 save_thumbnail_action (GPParams *p, const char *filename)
 {
-	return (save_file_to_file (p->camera, p->context, p->folder, filename,
-				   GP_FILE_TYPE_PREVIEW));
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_PREVIEW));
 }
 
 int
 save_raw_action (GPParams *p, const char *filename)
 {
-	return (save_file_to_file (p->camera, p->context, p->folder, filename,
-				   GP_FILE_TYPE_RAW));
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_RAW));
 }
 
 int
 save_audio_action (GPParams *p, const char *filename)
 {
-	return (save_file_to_file (p->camera, p->context, p->folder, filename,
-				   GP_FILE_TYPE_AUDIO));
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_AUDIO));
 }
 
 int
@@ -327,15 +375,24 @@ save_all_audio_action (GPParams *p, const char *filename)
 	/* not every file has an associated audio file */
 	if (camera_file_exists(p->camera, p->context, p->folder, filename,
 			       GP_FILE_TYPE_AUDIO))
-		return (save_file_to_file (p->camera, p->context, p->folder, filename,
+		return (save_file_to_file (p->camera, p->context, p->flags,
+					   p->folder, filename,
 					   GP_FILE_TYPE_AUDIO));
-	else
-		return GP_OK;
+	return GP_OK;
 }
 
 int
 delete_file_action (GPParams *p, const char *filename)
 {
+	if (p->flags & FLAGS_NEW) {
+		CameraFileInfo info;
+		
+		CR (gp_camera_file_get_info (p->camera, p->folder, filename,
+					     &info, p->context));
+		if (info.file.fields & GP_FILE_INFO_STATUS &&
+		    info.file.status == GP_FILE_STATUS_DOWNLOADED)
+			return (GP_OK);
+	}
 	return (gp_camera_file_delete (p->camera, p->folder, filename,
 				       p->context));
 }
@@ -368,11 +425,11 @@ print_hline (void)
         int i;
 
         for (i = 0; i < 20; i++)
-                printf ("-");
+                putchar ('-');
         printf ("+");
         for (i = 0; i < 59; i++)
-                printf ("-");
-        printf ("\n"); 
+                putchar ('-');
+        putchar ('\n'); 
 }
 #endif
 
@@ -400,12 +457,12 @@ print_exif_action (GPParams *p, const char *filename)
         }
 
         printf (_("EXIF tags:"));
-        printf ("\n");
+        putchar ('\n');
         print_hline ();
         printf ("%-20.20s", _("Tag"));
         printf ("|");
         printf ("%-59.59s", _("Value"));
-        printf ("\n");
+        putchar ('\n');
         print_hline ();
 #ifdef HAVE_EXIF_0_5_4
 	for (i = 0; i < EXIF_IFD_COUNT; i++)
@@ -427,7 +484,7 @@ print_exif_action (GPParams *p, const char *filename)
         if (ed->size) {
                 printf (_("EXIF data contains a thumbnail (%i bytes)."),
                         ed->size);
-                printf ("\n");
+                putchar ('\n');
         }
 
         exif_data_unref (ed);
@@ -449,12 +506,11 @@ list_cameras_action (GPParams *p)
 	r = gp_abilities_list_count (p->abilities_list);
 	if (r < 0)
 		return (r);
-	if (p->quiet)
-		fprintf (stdout, "%i\n", r);
+	if (p->flags & FLAGS_QUIET)
+		printf ("%i\n", r);
 	else {
-		fprintf (stdout, _("Number of supported "
-					"cameras: %i\n"), r);
-		fprintf (stdout, _("Supported cameras:\n"));
+		printf (_("Number of supported cameras: %i\n"), r);
+		printf (_("Supported cameras:\n"));
 	}
 	n = r;
 	for (i = 0; i < n; i++) {
@@ -462,8 +518,8 @@ list_cameras_action (GPParams *p)
 						     i, &a);
 		if (r < 0)
 			break;
-		if (p->quiet)
-			fprintf (stdout, "%s\n", a.model);
+		if (p->flags & FLAGS_QUIET)
+			printf ("%s\n", a.model);
 		else
 			switch (a.status) {
 			case GP_DRIVER_STATUS_TESTING:
@@ -502,12 +558,13 @@ list_ports_action (GPParams *p)
 		return (count);
 	}
 
-	if (!p->quiet) {
+	if (p->flags & FLAGS_QUIET)
+		printf("%i\n", count);
+	else {
 		printf(_("Devices found: %i\n"), count);
 		printf(_("Path                             Description\n"
 			"--------------------------------------------------------------\n"));
-	} else
-		printf("%i\n", count);
+	}
 
 	/* Now list the ports */
 	for (x = 0; x < count; x++) {
@@ -695,9 +752,8 @@ action_camera_about (GPParams *params)
 
 	CR (gp_camera_get_about (params->camera, &text, params->context));
 	
-	fprintf (stdout, _("About the camera driver:"));
-	fputc ('\n', stdout);
-	fprintf (stdout, "%s\n", _(text.text));
+	printf (_("About the camera driver:"));
+	printf ("\n%s\n", _(text.text));
 
 	return (GP_OK);
 }
@@ -709,9 +765,8 @@ action_camera_summary (GPParams *params)
 
 	CR (gp_camera_get_summary (params->camera, &text, params->context));
 
-	fprintf (stdout, _("Camera summary:"));
-	fputc ('\n', stdout);
-	fprintf (stdout, "%s\n", _(text.text));
+	printf (_("Camera summary:"));
+	printf ("\n%s\n", _(text.text));
 
 	return (GP_OK);
 }
@@ -723,9 +778,8 @@ action_camera_manual (GPParams *params)
 
 	CR (gp_camera_get_manual (params->camera, &text, params->context));
 
-	fprintf (stdout, _("Camera manual:"));
-	fputc ('\n', stdout);
-	fprintf (stdout, "%s\n", _(text.text));
+	printf (_("Camera manual:"));
+	printf ("\n%s\n", _(text.text));
 
 	return (GP_OK);
 }
@@ -738,7 +792,7 @@ action_camera_set_speed (GPParams *p, unsigned int speed)
 	/* Make sure we've got a serial port. */
 	CR (gp_camera_get_port_info (p->camera, &info));
 	if (info.type != GP_PORT_SERIAL) {
-		if (!p->quiet) {
+		if ((p->flags & FLAGS_QUIET) == 0) {
 			fprintf (stderr, _("You can only specify speeds for "
 					   "serial ports."));
 			fputc ('\n', stderr);
@@ -821,13 +875,12 @@ print_version_action (GPParams *p)
 	  CHECK_NULL (v[0]);
 	  printf ("%-17s %-12s ", name, v[0]);
 	  for (i = 1; v[i] != NULL; i++) {
-		  if (v[i+1] != NULL) {
+		  if (v[i+1] != NULL)
 			  printf ("%s, ", v[i]);
-		  } else {
+		  else
 			  printf ("%s", v[i]);
-		  }
 	  }
-	  printf ("\n");
+	  putchar ('\n');
 	}
 
 	return (GP_OK);
@@ -911,7 +964,7 @@ debug_func (GPLogLevel level, const char *domain, const char *format,
 	if (usec < 0) {sec--; usec += 1000000L;}
 	fprintf (stderr, "%li.%06li %s(%i): ", sec, usec, domain, level);
 	vfprintf (stderr, format, args);
-	fprintf (stderr, "\n");
+	fputc ('\n', stderr);
 }
 
 int
@@ -1001,16 +1054,15 @@ list_config_action (GPParams *p) {
 	return (GP_OK);
 }
 
-int
-get_config_action (GPParams *p, const char *name) {
-	CameraWidget *rootconfig,*child;
+static int
+_find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, CameraWidget **rootconfig) {
 	int	ret;
-	const char *label;
-	CameraWidgetType	type;
 
-	ret = gp_camera_get_config (p->camera, &rootconfig, p->context);
+	ret = gp_camera_get_config (p->camera, rootconfig, p->context);
 	if (ret != GP_OK) return ret;
-	ret = gp_widget_get_child_by_name (rootconfig, name, &child);
+	ret = gp_widget_get_child_by_name (*rootconfig, name, child);
+	if (ret != GP_OK) 
+		ret = gp_widget_get_child_by_label (*rootconfig, name, child);
 	if (ret != GP_OK) {
 		char		*part, *s, *newname;
 
@@ -1018,7 +1070,7 @@ get_config_action (GPParams *p, const char *name) {
 		if (!newname)
 			return GP_ERROR_NO_MEMORY;
 
-		child = rootconfig;
+		*child = *rootconfig;
 		part = newname;
 		while (part[0] == '/')
 			part++;
@@ -1028,10 +1080,12 @@ get_config_action (GPParams *p, const char *name) {
 			s = strchr (part,'/');
 			if (s)
 				*s='\0';
-			ret = gp_widget_get_child_by_name (child, part, &tmp);
+			ret = gp_widget_get_child_by_name (*child, part, &tmp);
+			if (ret != GP_OK)
+				ret = gp_widget_get_child_by_label (*child, part, &tmp);
 			if (ret != GP_OK)
 				break;
-			child = tmp;
+			*child = tmp;
 			if (!s) /* end of path */
 				break;
 			part = s+1;
@@ -1039,13 +1093,27 @@ get_config_action (GPParams *p, const char *name) {
 				part++;
 		}
 		if (s) { /* if we have stuff left over, we failed */
-			gp_context_error (p->context, _("%s not found in configuration tree.\n"), newname);
+			gp_context_error (p->context, _("%s not found in configuration tree."), newname);
 			free (newname);
-			gp_widget_free (rootconfig);
+			gp_widget_free (*rootconfig);
 			return GP_ERROR;
 		}
 		free (newname);
 	}
+	return GP_OK;
+}
+
+int
+get_config_action (GPParams *p, const char *name) {
+	CameraWidget *rootconfig,*child;
+	int	ret;
+	const char *label;
+	CameraWidgetType	type;
+
+	ret = _find_widget_by_name (p, name, &child, &rootconfig);
+	if (ret != GP_OK)
+		return ret;
+
 	ret = gp_widget_get_type (child, &type);
 	if (ret != GP_OK) {
 		gp_widget_free (rootconfig);
@@ -1067,7 +1135,7 @@ get_config_action (GPParams *p, const char *name) {
 			printf ("Type: TEXT\n"); /* parsed by scripts, no i18n */
 			printf ("Current: %s\n",txt);
 		} else {
-			gp_context_error (p->context, _("Failed to retrieve value of text widget %s.\n"), name);
+			gp_context_error (p->context, _("Failed to retrieve value of text widget %s."), name);
 		}
 		break;
 	}
@@ -1084,7 +1152,7 @@ get_config_action (GPParams *p, const char *name) {
 			printf ("Top: %g\n", t);	/* parsed by scripts, no i18n */
 			printf ("Step: %g\n", s);	/* parsed by scripts, no i18n */
 		} else {
-			gp_context_error (p->context, _("Failed to retrieve values of range widget %s.\n"), name);
+			gp_context_error (p->context, _("Failed to retrieve values of range widget %s."), name);
 		}
 		break;
 	}
@@ -1096,23 +1164,27 @@ get_config_action (GPParams *p, const char *name) {
 			printf ("Type: TOGGLE\n");
 			printf ("Current: %d\n",t);
 		} else {
-			gp_context_error (p->context, _("Failed to retrieve values of toggle widget %s.\n"), name);
+			gp_context_error (p->context, _("Failed to retrieve values of toggle widget %s."), name);
 		}
 		break;
 	}
 	case GP_WIDGET_DATE:  {		/* int			*/
-		int	t;
+		int	ret, t;
 		time_t	xtime;
+		struct tm *xtm;
+		char	timebuf[200];
 
 		ret = gp_widget_get_value (child, &t);
-		if (ret == GP_OK) {
-			printf ("Type: DATE\n");
-			printf ("Current: %d\n",t);
-			xtime = t;
-			printf ("Printable: %s\n",ctime (&xtime));
-		} else {
-			gp_context_error (p->context, _("Failed to retrieve values of date/time widget %s.\n"), name);
+		if (ret != GP_OK) {
+			gp_context_error (p->context, _("Failed to retrieve values of date/time widget %s."), name);
+			break;
 		}
+		xtime = t;
+		xtm = localtime (&xtime);
+		ret = strftime (timebuf, sizeof(timebuf), "%c", xtm);
+		printf ("Type: DATE\n");
+		printf ("Current: %d\n", t);
+		printf ("Printable: %s\n", timebuf);
 		break;
 	}
 	case GP_WIDGET_MENU:
@@ -1134,7 +1206,7 @@ get_config_action (GPParams *p, const char *name) {
 				printf ("Choice: %d %s\n", i, choice);
 			}
 		} else {
-			gp_context_error (p->context, _("Failed to retrieve values of radio widget %s.\n"), name);
+			gp_context_error (p->context, _("Failed to retrieve values of radio widget %s."), name);
 		}
 		break;
 	}
@@ -1148,3 +1220,154 @@ get_config_action (GPParams *p, const char *name) {
 	gp_widget_free (rootconfig);
 	return (GP_OK);
 }
+
+int
+set_config_action (GPParams *p, const char *name, const char *value) {
+	CameraWidget *rootconfig,*child;
+	int	ret;
+	const char *label;
+	CameraWidgetType	type;
+
+	ret = _find_widget_by_name (p, name, &child, &rootconfig);
+	if (ret != GP_OK)
+		return ret;
+
+	ret = gp_widget_get_type (child, &type);
+	if (ret != GP_OK) {
+		gp_widget_free (rootconfig);
+		return ret;
+	}
+	ret = gp_widget_get_label (child, &label);
+	if (ret != GP_OK) {
+		gp_widget_free (rootconfig);
+		return ret;
+	}
+
+	switch (type) {
+	case GP_WIDGET_TEXT: {		/* char *		*/
+		ret = gp_widget_set_value (child, value);
+		if (ret != GP_OK)
+			gp_context_error (p->context, _("Failed to set the value of text widget %s to %s."), name, value);
+		break;
+	}
+	case GP_WIDGET_RANGE: {	/* float		*/
+		float	f,t,b,s;
+
+		ret = gp_widget_get_range (child, &b, &t, &s);
+		if (ret != GP_OK)
+			break;
+		if (!sscanf (value, "%f", &f)) {
+			gp_context_error (p->context, _("The passed value %s is not a floating point value."), value);
+			ret = GP_ERROR_BAD_PARAMETERS;
+			break;
+		}
+		if ((f < b) || (f > t)) {
+			gp_context_error (p->context, _("The passed value %f is not within the expected range %f - %f."), f, b, t);
+			ret = GP_ERROR_BAD_PARAMETERS;
+			break;
+		}
+		ret = gp_widget_set_value (child, &f);
+		if (ret != GP_OK)
+			gp_context_error (p->context, _("Failed to set the value of range widget %s to %f."), name, f);
+		break;
+	}
+	case GP_WIDGET_TOGGLE: {	/* int		*/
+		int	t;
+
+		t = 2;
+		if (	strcasecmp (value, "off")	|| strcasecmp (value, "no")	||
+			strcasecmp (value, "false")	|| strcmp (value, "0")		||
+			strcasecmp (value, _("off"))	|| strcasecmp (value, _("no"))	||
+			strcasecmp (value, _("false"))
+		)
+			t = 0;
+		if (	strcasecmp (value, "on")	|| strcasecmp (value, "yes")	||
+			strcasecmp (value, "true")	|| strcmp (value, "0")		||
+			strcasecmp (value, _("on"))	|| strcasecmp (value, _("yes"))	||
+			strcasecmp (value, _("true"))
+		)
+			t = 1;
+		if (t == 2) {
+			gp_context_error (p->context, _("The passed value %s is not a valid toggle value."), value);
+			ret = GP_ERROR_BAD_PARAMETERS;
+			break;
+		}
+		ret = gp_widget_set_value (child, &t);
+		if (ret != GP_OK)
+			gp_context_error (p->context, _("Failed to set values %s of toggle widget %s."), value, name);
+		break;
+	}
+	case GP_WIDGET_DATE:  {		/* int			*/
+		int	t = -1;
+		struct tm xtm;
+
+#ifdef HAVE_STRPTIME
+		if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
+			t = mktime (&xtm);
+#endif
+		if (t == -1) {
+			if (!sscanf (value, "%d", &t)) {
+				gp_context_error (p->context, _("The passed value %s is neither a valid time nor an integer."), value);
+				ret = GP_ERROR_BAD_PARAMETERS;
+				break;
+			}
+		}
+		ret = gp_widget_set_value (child, &t);
+		if (ret != GP_OK)
+			gp_context_error (p->context, _("Failed to set new time of date/time widget %s to %s."), name, value);
+		break;
+	}
+	case GP_WIDGET_MENU:
+	case GP_WIDGET_RADIO: { /* char *		*/
+		int cnt, i;
+
+		cnt = gp_widget_count_choices (child);
+		if (cnt < GP_OK) {
+			ret = cnt;
+			break;
+		}
+		ret = GP_ERROR_BAD_PARAMETERS;
+		for ( i=0; i<cnt; i++) {
+			const char *choice;
+
+			ret = gp_widget_get_choice (child, i, &choice);
+			if (ret != GP_OK)
+				continue;
+			if (!strcmp (choice, value)) {
+				ret = gp_widget_set_value (child, value);
+				break;
+			}
+		}
+		if (i != cnt)
+			break;
+
+		if (sscanf (value, "%d", &i)) {
+			if ((i>= 0) && (i < cnt)) {
+				const char *choice;
+
+				ret = gp_widget_get_choice (child, i, &choice);
+				if (ret == GP_OK)
+					ret = gp_widget_set_value (child, choice);
+				break;
+			}
+		}
+		gp_context_error (p->context, _("Choice %s not found within list of choices."), value);
+		break;
+	}
+
+	/* ignore: */
+	case GP_WIDGET_WINDOW:
+	case GP_WIDGET_SECTION:
+	case GP_WIDGET_BUTTON:
+		gp_context_error (p->context, _("The %s widget is not configurable."), name);
+		ret = GP_ERROR_BAD_PARAMETERS;
+		break;
+	}
+	if (ret == GP_OK) {
+		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (ret != GP_OK)
+			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
+	}
+	gp_widget_free (rootconfig);
+	return (ret);
+}
